import { ChatCompletion, ChatCompletionChunk, ChatCompletionCreateParamsBase } from "openai/resources/chat/completions";
import * as Core from "openai/core";
import { CreateEmbeddingResponse, EmbeddingCreateParams } from "openai/resources/embeddings";
import { Stream } from "openai/streaming";
import { Logger } from "./Logger";
import { ModerationCreateParams, ModerationCreateResponse } from "openai/resources/moderations";
export interface IAIClient {
    createChatCompletion(createChatCompletionRequest: ChatCompletionCreateParamsBase, options?: Core.RequestOptions, logMeta?: {
        [key: string]: string;
    }): Promise<ChatCompletion>;
    createStreamingChatCompletion(createChatCompletionRequest: ChatCompletionCreateParamsBase, options?: Core.RequestOptions, logMeta?: {
        [key: string]: string;
    }): Promise<Stream<ChatCompletionChunk>>;
    performEmbedding(createEmbeddingRequest: EmbeddingCreateParams, options?: Core.RequestOptions, logMeta?: {
        [key: string]: string;
    }): Promise<CreateEmbeddingResponse>;
    performModeration(body: ModerationCreateParams, options?: Core.RequestOptions, logMeta?: {
        [key: string]: string;
    }): Promise<ModerationCreateResponse>;
}
export interface EmbeddingClient {
    performEmbedding(createEmbeddingRequest: EmbeddingCreateParams, options?: Core.RequestOptions): Core.APIPromise<CreateEmbeddingResponse>;
}
export interface ModerationClient {
    performModeration(body: ModerationCreateParams, options?: Core.RequestOptions): Core.APIPromise<ModerationCreateResponse>;
}
export interface EnrichmentClient {
    createChatCompletion(createChatCompletionRequest: ChatCompletionCreateParamsBase, options?: Core.RequestOptions): Core.APIPromise<ChatCompletion>;
    createStreamingChatCompletion(createChatCompletionRequest: ChatCompletionCreateParamsBase, options?: Core.RequestOptions): Core.APIPromise<Stream<ChatCompletionChunk>>;
}
type ClientProviders = EmbeddingClient | EnrichmentClient | ModerationClient;
interface ClientProvider<T extends ClientProviders> {
    clients: T[];
    models: string[];
    clientPoolSize?: number;
}
export declare class AIClient implements IAIClient {
    private logger?;
    private embeddingPoolMap;
    private moderationPoolMap;
    private enrichmentPoolMap;
    constructor(embeddingProviders: ClientProvider<EmbeddingClient>[], moderationProviders: ClientProvider<ModerationClient>[], enrichmentProviders: ClientProvider<EnrichmentClient>[], logger?: Logger | undefined);
    private initializePoolMap;
    performEmbedding(createEmbeddingRequest: EmbeddingCreateParams, options?: Core.RequestOptions, logMeta?: {
        [key: string]: string;
    }): Promise<CreateEmbeddingResponse>;
    performModeration(body: ModerationCreateParams, options?: Core.RequestOptions, logMeta?: {
        [key: string]: string;
    }): Promise<ModerationCreateResponse>;
    createChatCompletion(createChatCompletionRequest: ChatCompletionCreateParamsBase, options?: Core.RequestOptions, logMeta?: {
        [key: string]: string;
    }): Promise<ChatCompletion>;
    createStreamingChatCompletion(createChatCompletionRequest: ChatCompletionCreateParamsBase, options?: Core.RequestOptions, logMeta?: {
        [key: string]: string;
    }): Promise<Stream<ChatCompletionChunk>>;
}
export {};
