/// <reference types="node" />
import { JSONSchemaType } from "ajv";
import { GPTModels } from "./gptUtils";
import { IAuditor } from "./IAuditor";
import { Logger } from "./Logger";
import ReadableStream = NodeJS.ReadableStream;
import { ChatCompletion, ChatCompletionCreateParamsBase } from "openai/resources/chat/completions";
import { Embedding, EmbeddingCreateParams } from "openai/resources/embeddings";
import { AIClient } from "./AIClient";
import { ICacher } from "./ICacher";
import { ModerationCreateParams } from "openai/resources/moderations";
import * as Core from "openai/core";
import { ChatCompletionAssistantMessageParam, ChatCompletionSystemMessageParam, ChatCompletionToolMessageParam, ChatCompletionUserMessageParam } from "openai/resources/chat/completions";
type MetaDataType = {
    [k: string]: string;
};
export type ChatCompletionFunctionsWithTypes2<T, N extends string> = {
    readonly name: N;
    readonly description: string;
    readonly parameters: JSONSchemaType<Wrapper<N, T>>;
    readonly partialStreamPath?: string[];
};
export type Result2<T, N extends string> = {
    readonly name: N;
    readonly entry: ChatCompletionFunctionsWithTypes2<T, N>;
    readonly value: T;
};
export type ChatCompletionFunctionsWithTypes<T, N extends string> = {
    readonly name: N;
    readonly description: string;
    readonly parameters: JSONSchemaType<T>;
    readonly partialStreamPath?: string[];
};
export type Result<T, N extends string> = {
    readonly name: N;
    readonly entry: ChatCompletionFunctionsWithTypes<T, N>;
    readonly value: T;
};
export declare const defaultStreamDelimiterSeparator = "|{-*-}|";
export declare class Contractor<MetaData extends MetaDataType> {
    private aiClient;
    private functionsMessagePlaceHolder;
    private auditor?;
    private cacher?;
    private maxTokensPerRequest;
    private streamObjectSeparator;
    private logger?;
    private settings?;
    private readonly schemaValidationCache;
    constructor(aiClient: AIClient, functionsMessagePlaceHolder: string, auditor?: IAuditor<MetaData> | undefined, cacher?: ICacher | undefined, maxTokensPerRequest?: number, streamObjectSeparator?: string, logger?: Logger | undefined, settings?: {
        disableModeration?: boolean | undefined;
    } | undefined);
    streamingFunction<T1, N1 extends string, OUT>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, functions: [ChatCompletionFunctionsWithTypes<T1, N1>], transformObjectStream: (streamingObject: Result<T1, N1>) => Promise<OUT>, responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<NodeJS.ReadableStream | undefined>;
    streamingFunction<T1, T2, N1 extends string, N2 extends string, OUT>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>], transformObjectStream: (streamingObject: Result<T1, N1> | Result<T2, N2>) => Promise<OUT>, responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<NodeJS.ReadableStream | undefined>;
    streamingFunction<T1, T2, T3, N1 extends string, N2 extends string, N3 extends string, OUT>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>], transformObjectStream: (streamingObject: Result<T1, N1> | Result<T2, N2> | Result<T3, N3>) => Promise<OUT>, responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<NodeJS.ReadableStream | undefined>;
    streamingFunction<T1, T2, T3, T4, N1 extends string, N2 extends string, N3 extends string, N4 extends string, OUT>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>], transformObjectStream: (streamingObject: Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4>) => Promise<OUT>, responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<NodeJS.ReadableStream | undefined>;
    streamingFunction<T1, T2, T3, T4, T5, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string, OUT>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>], transformObjectStream: (streamingObject: Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5>) => Promise<OUT>, responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<NodeJS.ReadableStream | undefined>;
    streamingFunction<T1, T2, T3, T4, T5, T6, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string, N6 extends string, OUT>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>, ChatCompletionFunctionsWithTypes<T6, N6>], transformObjectStream: (streamingObject: Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5> | Result<T6, N6>) => Promise<OUT>, responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<NodeJS.ReadableStream | undefined>;
    streamingFunction<T1, T2, T3, T4, T5, T6, T7, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string, N6 extends string, N7 extends string, OUT>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>, ChatCompletionFunctionsWithTypes<T6, N6>, ChatCompletionFunctionsWithTypes<T7, N7>], transformObjectStream: (streamingObject: Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5> | Result<T6, N6> | Result<T7, N7>) => Promise<OUT>, responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<NodeJS.ReadableStream | undefined>;
    streamingFunction<T1, T2, T3, T4, T5, T6, T7, T8, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string, N6 extends string, N7 extends string, N8 extends string, OUT>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>, ChatCompletionFunctionsWithTypes<T6, N6>, ChatCompletionFunctionsWithTypes<T7, N7>, ChatCompletionFunctionsWithTypes<T8, N8>], transformObjectStream: (streamingObject: Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5> | Result<T6, N6> | Result<T7, N7> | Result<T8, N8>) => Promise<OUT>, responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<NodeJS.ReadableStream | undefined>;
    streamingFunction<T1, T2, T3, T4, T5, T6, T7, T8, T9, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string, N6 extends string, N7 extends string, N8 extends string, N9 extends string, OUT>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>, ChatCompletionFunctionsWithTypes<T6, N6>, ChatCompletionFunctionsWithTypes<T7, N7>, ChatCompletionFunctionsWithTypes<T8, N8>, ChatCompletionFunctionsWithTypes<T9, N9>], transformObjectStream: (streamingObject: Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5> | Result<T6, N6> | Result<T7, N7> | Result<T8, N8> | Result<T9, N9>) => Promise<OUT>, responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<NodeJS.ReadableStream | undefined>;
    streamingFunction<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string, N6 extends string, N7 extends string, N8 extends string, N9 extends string, N10 extends string, OUT>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>, ChatCompletionFunctionsWithTypes<T6, N6>, ChatCompletionFunctionsWithTypes<T7, N7>, ChatCompletionFunctionsWithTypes<T8, N8>, ChatCompletionFunctionsWithTypes<T9, N9>, ChatCompletionFunctionsWithTypes<T10, N10>], transformObjectStream: (streamingObject: Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5> | Result<T6, N6> | Result<T7, N7> | Result<T8, N8> | Result<T9, N9> | Result<T10, N10>) => Promise<OUT>, responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<NodeJS.ReadableStream | undefined>;
    streamingHealer(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, healer: (streamStr: string) => (string | undefined), responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletion>, maxTokens?: number, manipulateResult?: (input: {
        healedStream: string;
    }) => any, onSuccessFinished?: () => void): Promise<NodeJS.ReadableStream | undefined>;
    makeBlockingRequestWithFunctions<T1, N1 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result<T1, N1> | undefined>;
    makeBlockingRequestWithFunctions<T1, T2, N1 extends string, N2 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result<T1, N1> | Result<T2, N2> | undefined>;
    makeBlockingRequestWithFunctions<T1, T2, T3, N1 extends string, N2 extends string, N3 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | undefined>;
    makeBlockingRequestWithFunctions<T1, T2, T3, T4, N1 extends string, N2 extends string, N3 extends string, N4 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | undefined>;
    makeBlockingRequestWithFunctions<T1, T2, T3, T4, T5, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5> | undefined>;
    makeBlockingRequestWithFunctions<T1, T2, T3, T4, T5, T6, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string, N6 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>, ChatCompletionFunctionsWithTypes<T6, N6>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5> | Result<T6, N6> | undefined>;
    makeBlockingRequestWithFunctions<T1, T2, T3, T4, T5, T6, T7, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string, N6 extends string, N7 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>, ChatCompletionFunctionsWithTypes<T6, N6>, ChatCompletionFunctionsWithTypes<T7, N7>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5> | Result<T6, N6> | Result<T7, N7> | undefined>;
    makeBlockingRequestWithFunctions<T1, T2, T3, T4, T5, T6, T7, T8, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string, N6 extends string, N7 extends string, N8 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>, ChatCompletionFunctionsWithTypes<T6, N6>, ChatCompletionFunctionsWithTypes<T7, N7>, ChatCompletionFunctionsWithTypes<T8, N8>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5> | Result<T6, N6> | Result<T7, N7> | Result<T8, N8> | undefined>;
    makeBlockingRequestWithFunctions<T1, T2, T3, T4, T5, T6, T7, T8, T9, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string, N6 extends string, N7 extends string, N8 extends string, N9 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>, ChatCompletionFunctionsWithTypes<T6, N6>, ChatCompletionFunctionsWithTypes<T7, N7>, ChatCompletionFunctionsWithTypes<T8, N8>, ChatCompletionFunctionsWithTypes<T9, N9>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5> | Result<T6, N6> | Result<T7, N7> | Result<T8, N8> | Result<T9, N9> | undefined>;
    makeBlockingRequestWithFunctions<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string, N6 extends string, N7 extends string, N8 extends string, N9 extends string, N10 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>, ChatCompletionFunctionsWithTypes<T6, N6>, ChatCompletionFunctionsWithTypes<T7, N7>, ChatCompletionFunctionsWithTypes<T8, N8>, ChatCompletionFunctionsWithTypes<T9, N9>, ChatCompletionFunctionsWithTypes<T10, N10>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result<T1, N1> | Result<T2, N2> | Result<T3, N3> | Result<T4, N4> | Result<T5, N5> | Result<T6, N6> | Result<T7, N7> | Result<T8, N8> | Result<T9, N9> | Result<T10, N10> | undefined>;
    makeBlockingRequestWithFunctionsAlt<T1, N1 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result2<T1, N1> | undefined>;
    makeBlockingRequestWithFunctionsAlt<T1, T2, N1 extends string, N2 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result2<T1, N1> | Result2<T2, N2> | undefined>;
    makeBlockingRequestWithFunctionsAlt<T1, T2, T3, N1 extends string, N2 extends string, N3 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result2<T1, N1> | Result2<T2, N2> | Result2<T3, N3> | undefined>;
    makeBlockingRequestWithFunctionsAlt<T1, T2, T3, T4, N1 extends string, N2 extends string, N3 extends string, N4 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result2<T1, N1> | Result2<T2, N2> | Result2<T3, N3> | Result2<T4, N4> | undefined>;
    makeBlockingRequestWithFunctionsAlt<T1, T2, T3, T4, T5, N1 extends string, N2 extends string, N3 extends string, N4 extends string, N5 extends string>(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, functions: [ChatCompletionFunctionsWithTypes<T1, N1>, ChatCompletionFunctionsWithTypes<T2, N2>, ChatCompletionFunctionsWithTypes<T3, N3>, ChatCompletionFunctionsWithTypes<T4, N4>, ChatCompletionFunctionsWithTypes<T5, N5>], responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<Result2<T1, N1> | Result2<T2, N2> | Result2<T3, N3> | Result2<T4, N4> | Result2<T5, N5> | undefined>;
    makeBlockingRequest(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, actionName: string, responseSize?: number, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<string>;
    makeStreamingRequest(systemMessage: string, messages: RequestMessageFormat[], model: GPTModels, responseSize?: number, functions?: Array<ChatCompletionFunctionsWithTypes<any, any>>, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, maxTokens?: number): Promise<ReadableStream | undefined>;
    performModeration(body: ModerationCreateParams, options?: Core.RequestOptions, logMetaData?: MetaData): Promise<void>;
    getStreamSeparator: () => string;
    getFunctionsMessagePlaceHolder: () => string;
    private moderateLastMessage;
    singleFunction: <T>(systemMessage: string, messages: RequestMessageFormat[], model: string | undefined, gptFunction: {
        name: string;
        description: string;
        parameters: JSONSchemaType<T>;
    }, logMetaData?: MetaData, requestOverrides?: Partial<ChatCompletionCreateParamsBase>, responseSize?: number, maxTokens?: number) => Promise<T | undefined>;
    performEmbedding: (inputContent: EmbeddingCreateParams['input'], userId: string, model: 'text-embedding-ada-002' | 'text-embedding-3-small' | string, logMetaData?: MetaData) => Promise<Embedding[]>;
    private extractFunctionValidatedResult;
    private recordAudit;
    private measureRequest;
}
interface Wrapper<N extends string, T> {
    type: N;
    value: T;
}
export declare const createResultWrapper: <N extends string, T>(name: N, tSchema: JSONSchemaType<T>, description?: string) => JSONSchemaType<Wrapper<N, T>>;
export declare const createResultsWrapper: (funcs: ({
    name: string;
    parameters: JSONSchemaType<any>;
    description: string;
})[]) => JSONSchemaType<any>;
export type RequestMessageFormat = ChatCompletionSystemMessageParam | ChatCompletionUserMessageParam | ChatCompletionAssistantMessageParam | ChatCompletionToolMessageParam;
export {};
