"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createResultsWrapper = exports.createResultWrapper = exports.Contractor = exports.defaultStreamDelimiterSeparator = void 0;
const stream_1 = require("stream");
const gptUtils_1 = require("./gptUtils");
const utils_1 = require("./utils");
const StreamListenerTransform_1 = require("./StreamListenerTransform");
const SchemaValidationCache_1 = require("./SchemaValidationCache");
const ThrottleTransform_1 = require("./ThrottleTransform");
const OpenAIStreamToStreamedObjectTransform_1 = require("./OpenAIStreamToStreamedObjectTransform");
const StreamMITMTransform_1 = require("./StreamMITMTransform");
const JSON5 = __importStar(require("./json5"));
const JSONSchemaToTypescriptConverter_1 = require("./JSONSchemaToTypescriptConverter");
const OpenAIStreamToStreamedHealedTransform_1 = require("./OpenAIStreamToStreamedHealedTransform");
const OpenAIStreamChunkTransform_1 = require("./OpenAIStreamChunkTransform");
exports.defaultStreamDelimiterSeparator = '|{-*-}|';
class Contractor {
    constructor(aiClient, functionsMessagePlaceHolder, auditor, cacher, maxTokensPerRequest = 8000, streamObjectSeparator = exports.defaultStreamDelimiterSeparator, logger, settings) {
        this.aiClient = aiClient;
        this.functionsMessagePlaceHolder = functionsMessagePlaceHolder;
        this.auditor = auditor;
        this.cacher = cacher;
        this.maxTokensPerRequest = maxTokensPerRequest;
        this.streamObjectSeparator = streamObjectSeparator;
        this.logger = logger;
        this.settings = settings;
        this.getStreamSeparator = () => this.streamObjectSeparator;
        this.getFunctionsMessagePlaceHolder = () => this.functionsMessagePlaceHolder;
        this.singleFunction = (systemMessage, messages, model = 'gpt-3.5-turbo-0613', gptFunction, logMetaData, requestOverrides, responseSize = 2000, maxTokens = this.maxTokensPerRequest) => __awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
            const { promptSize } = this.measureRequest(model, systemMessage, messages, responseSize, maxTokens);
            yield this.moderateLastMessage(messages);
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.info(`using gpt function [${gptFunction.name}]`, logMetaData);
            const request = Object.assign(Object.assign({ model: promptSize + responseSize > 4000 ? (0, gptUtils_1.largeModel)(model) : model, messages: [
                    { role: 'system', content: systemMessage },
                    ...messages,
                ], temperature: 0, max_tokens: responseSize, functions: [gptFunction] }, requestOverrides), { function_call: { name: gptFunction.name } });
            let result = undefined;
            try {
                result = (yield this.aiClient.createChatCompletion(request, logMetaData));
                const readContent = (_d = (_c = (_b = result.choices[0]) === null || _b === void 0 ? void 0 : _b.message) === null || _c === void 0 ? void 0 : _c.function_call) === null || _d === void 0 ? void 0 : _d.arguments;
                const validatedResult = this.extractFunctionValidatedResult(readContent, gptFunction.parameters);
                yield this.recordAudit(systemMessage, messages, result.model, readContent !== null && readContent !== void 0 ? readContent : '', request, result, gptFunction.name, undefined, logMetaData);
                return validatedResult;
            }
            catch (err) {
                (_e = this.logger) === null || _e === void 0 ? void 0 : _e.error(`error performing [${gptFunction.name}]`, err);
                yield this.recordAudit(systemMessage, messages, (_f = result === null || result === void 0 ? void 0 : result.model) !== null && _f !== void 0 ? _f : model, (_k = (_j = (_h = (_g = result === null || result === void 0 ? void 0 : result.choices[0]) === null || _g === void 0 ? void 0 : _g.message) === null || _h === void 0 ? void 0 : _h.function_call) === null || _j === void 0 ? void 0 : _j.arguments) !== null && _k !== void 0 ? _k : '', request, result, gptFunction.name, undefined, logMetaData);
                throw new Error(`error performing single function [${gptFunction.name}]`);
            }
        });
        this.performEmbedding = (inputContent, userId, model, logMetaData) => __awaiter(this, void 0, void 0, function* () {
            var _l, _m, _o, _p;
            const createEmbeddingRequest = {
                input: inputContent,
                model,
                user: userId
            };
            const result = (_l = (yield (0, utils_1.truthy)(this.cacher, (_) => __awaiter(this, void 0, void 0, function* () { return yield _.retrieveEmbeddingFromCache(createEmbeddingRequest, logMetaData); })))) !== null && _l !== void 0 ? _l : yield this.aiClient.performEmbedding(createEmbeddingRequest, logMetaData);
            yield ((_m = this.auditor) === null || _m === void 0 ? void 0 : _m.auditRequest({
                request: createEmbeddingRequest,
                resultRaw: result,
                result: { data: { content: result.data } },
                requestType: 'embedding',
                requestSig: (_o = logMetaData === null || logMetaData === void 0 ? void 0 : logMetaData['requestSig']) !== null && _o !== void 0 ? _o : '-',
                metaData: logMetaData,
            }));
            if (!result.isFromCache) {
                (_p = this.cacher) === null || _p === void 0 ? void 0 : _p.cacheEmbedding(result, createEmbeddingRequest, logMetaData).then(_ignore => ({}));
            }
            return result.data;
        });
        this.schemaValidationCache = new SchemaValidationCache_1.SchemaValidationCache();
    }
    streamingFunction(systemMessage, messages, model, functions, transformObjectStream, responseSize = 800, logMetaData, requestOverrides, maxTokens = this.maxTokensPerRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            const regexp = new RegExp(this.functionsMessagePlaceHolder, 'g');
            const responseFormatGen = new JSONSchemaToTypescriptConverter_1.JSONSchemaToTypescriptConverter((0, exports.createResultsWrapper)(functions), 'Result').generateTypescript();
            const stream = yield this.makeStreamingRequest(systemMessage.replace(regexp, responseFormatGen), messages.map(m => m.role === 'user'
                ? (Object.assign(Object.assign({}, m), { content: typeof m.content === 'string' ? m.content.replace(regexp, responseFormatGen) : m.content })) : m), model, responseSize, functions, logMetaData, requestOverrides, maxTokens);
            if (!stream) {
                return undefined;
            }
            const throttler = new ThrottleTransform_1.ThrottledTransform({
                name: 'task creation thr',
                flushDebounceTimeMs: 1000,
                maxIdleTimeoutMs: 20000,
                windowSize: 50,
            });
            const validators = new Map(functions.map(({ name, parameters, description }) => ([name, this.schemaValidationCache.getValidator(parameters)])));
            const objectTransform = new OpenAIStreamToStreamedObjectTransform_1.OpenAIStreamToStreamedObjectTransform(validators, new Map(functions.filter(_ => !!_.partialStreamPath)
                .map(_ => { var _a; return [_.name, ['value', ...((_a = _.partialStreamPath) !== null && _a !== void 0 ? _a : [])]]; })), this.logger, 'name', 'value');
            const objectStreamTransformer = new StreamMITMTransform_1.StreamMITMTransform((input, functionName) => __awaiter(this, void 0, void 0, function* () {
                if (!functionName) {
                    throw new Error('action did not result in a function, try to ask for specific function or direct AI to answer by using any of the available functions');
                }
                return yield transformObjectStream({
                    name: functionName,
                    entry: functions.find(_ => _.name === functionName),
                    value: input.value,
                });
            }), this.streamObjectSeparator);
            return (0, stream_1.pipeline)(stream, throttler, objectTransform, objectStreamTransformer, err => {
                var _a;
                if (err) {
                    (_a = this.logger) === null || _a === void 0 ? void 0 : _a.error('error while processing stream interim results', err, logMetaData);
                }
            });
        });
    }
    streamingHealer(systemMessage, messages, model, healer, responseSize = 800, logMetaData, requestOverrides, maxTokens = this.maxTokensPerRequest, manipulateResult, onSuccessFinished) {
        return __awaiter(this, void 0, void 0, function* () {
            const stream = yield this.makeStreamingRequest(systemMessage, messages, model, responseSize, undefined, logMetaData, requestOverrides, maxTokens);
            if (!stream) {
                return undefined;
            }
            const throttler = new ThrottleTransform_1.ThrottledTransform({
                name: 'task creation thr',
                flushDebounceTimeMs: 1000,
                maxIdleTimeoutMs: 20000,
                windowSize: 10,
            });
            const healedTransform = new OpenAIStreamToStreamedHealedTransform_1.OpenAIStreamToStreamedHealedTransform(healer, this.logger);
            const objectStreamTransformer = new StreamMITMTransform_1.StreamMITMTransform((input) => __awaiter(this, void 0, void 0, function* () {
                return manipulateResult ? manipulateResult({ healedStream: input }) : { healedStream: manipulateResult };
            }), this.streamObjectSeparator, onSuccessFinished);
            return (0, stream_1.pipeline)(stream, throttler, healedTransform, objectStreamTransformer, err => {
                var _a;
                if (err) {
                    (_a = this.logger) === null || _a === void 0 ? void 0 : _a.error('error while processing stream interim results', err, logMetaData);
                }
            });
        });
    }
    makeBlockingRequestWithFunctions(systemMessage, messages, model, actionName, functions, responseSize = 800, logMetaData, requestOverrides, maxTokens = this.maxTokensPerRequest) {
        var _a, _b, _c, _d, _e, _f, _g;
        return __awaiter(this, void 0, void 0, function* () {
            const { promptSize } = this.measureRequest(model, systemMessage, messages, responseSize, maxTokens);
            yield this.moderateLastMessage(messages);
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.info(`performing blocking request [${actionName}]`, logMetaData);
            const request = Object.assign(Object.assign({ model: promptSize + responseSize > 4000 ? (0, gptUtils_1.largeModel)(model) : model, messages: [
                    { role: 'system', content: systemMessage },
                    ...messages,
                ], temperature: 0, max_tokens: responseSize }, requestOverrides), { functions: ((_b = functions === null || functions === void 0 ? void 0 : functions.length) !== null && _b !== void 0 ? _b : 0) > 0 ? functions : undefined, function_call: ((_c = functions === null || functions === void 0 ? void 0 : functions.length) !== null && _c !== void 0 ? _c : 0) > 1 ? 'auto' : undefined });
            let result = undefined;
            try {
                result = (yield this.aiClient.createChatCompletion(request, logMetaData));
                const funCall = (_e = (_d = result.choices[0]) === null || _d === void 0 ? void 0 : _d.message) === null || _e === void 0 ? void 0 : _e.function_call;
                (0, utils_1.assertIsDefined)(funCall, 'function was not returned');
                const validator = functions.find(_ => _.name === funCall.name);
                (0, utils_1.assertIsDefined)(validator, `could not find func validator for name [${funCall.name}]`);
                const validatedResult = this.extractFunctionValidatedResult(funCall.arguments, validator.parameters);
                yield this.recordAudit(systemMessage, messages, model, (_f = funCall.arguments) !== null && _f !== void 0 ? _f : '', request, result, actionName, undefined, logMetaData);
                return {
                    name: funCall.name,
                    entry: validator,
                    value: validatedResult,
                };
            }
            catch (err) {
                (_g = this.logger) === null || _g === void 0 ? void 0 : _g.error(`error performing [${actionName}]`, err);
                yield this.recordAudit(systemMessage, messages, model, "", request, result, actionName, err, logMetaData);
                throw new Error("error performing blocking request with functions");
            }
        });
    }
    makeBlockingRequestWithFunctionsAlt(systemMessage, messages, model, actionName, functions, responseSize = 800, logMetaData, requestOverrides, maxTokens = this.maxTokensPerRequest) {
        var _a, _b, _c, _d, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            const regexp = new RegExp(this.functionsMessagePlaceHolder, 'g');
            const responseFormatGen = new JSONSchemaToTypescriptConverter_1.JSONSchemaToTypescriptConverter((0, exports.createResultsWrapper)(functions), 'Result').generateTypescript();
            const _messages = messages.map(m => m.role === 'user'
                ? (Object.assign(Object.assign({}, m), { content: typeof m.content === 'string' ? m.content.replace(regexp, responseFormatGen) : m.content }))
                : m);
            const { promptSize } = this.measureRequest(model, systemMessage, _messages, responseSize, maxTokens);
            yield this.moderateLastMessage(_messages);
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.info(`performing blocking request alt [${actionName}]`, logMetaData);
            const request = Object.assign({ model: promptSize + responseSize > 4000 ? (0, gptUtils_1.largeModel)(model) : model, messages: [
                    { role: 'system', content: systemMessage.replace(regexp, responseFormatGen) },
                    ..._messages,
                ], temperature: 0, max_tokens: responseSize }, requestOverrides);
            let result = undefined;
            try {
                result = (_b = yield (0, utils_1.truthy)(this.cacher, (_) => __awaiter(this, void 0, void 0, function* () { return yield _.retrieveRequestFromCache(request, logMetaData); }))) !== null && _b !== void 0 ? _b : yield this.aiClient.createChatCompletion(request, logMetaData);
                const objStr = (_d = (_c = result.choices[0]) === null || _c === void 0 ? void 0 : _c.message) === null || _d === void 0 ? void 0 : _d.content;
                (0, utils_1.assertIsDefined)(objStr, 'response content not generated');
                const objJson = JSON5.parse(objStr, undefined, (error, stack, root) => {
                    if (error.message.includes(`invalid character '`)) {
                        return { type: 'skip-char' };
                    }
                    else {
                        return undefined;
                    }
                });
                // const objJson = JSON5.parse(objStr, );
                const objType = objJson['name'];
                (0, utils_1.assertIsDefined)(objType, 'could not resolve object type');
                const objValueStr = JSON.stringify(objJson['value']);
                const validator = functions.find(_ => _.name === objType);
                (0, utils_1.assertIsDefined)(validator, `could not find func validator for name [${objType}]`);
                const validatedResult = this.extractFunctionValidatedResult(objValueStr, validator.parameters);
                if (!result.isFromCache) {
                    (_e = this.cacher) === null || _e === void 0 ? void 0 : _e.cacheRequest(result, request, logMetaData).then(_ignore => ({}));
                }
                yield this.recordAudit(systemMessage, _messages, model, objValueStr, request, result, actionName, undefined, logMetaData);
                return {
                    name: objType,
                    entry: validator,
                    value: validatedResult,
                };
            }
            catch (err) {
                (_f = this.logger) === null || _f === void 0 ? void 0 : _f.error(`error performing [${actionName}]`, err);
                yield this.recordAudit(systemMessage, _messages, model, "", request, result, undefined, err, logMetaData);
                throw new Error("error performing blocking request with functions alt");
            }
        });
    }
    makeBlockingRequest(systemMessage, messages, model, actionName, responseSize = 800, logMetaData, requestOverrides, maxTokens = this.maxTokensPerRequest) {
        var _a, _b, _c, _d, _e;
        return __awaiter(this, void 0, void 0, function* () {
            const { promptSize } = this.measureRequest(model, systemMessage, messages, responseSize, maxTokens);
            yield this.moderateLastMessage(messages);
            const oaiModel = promptSize + responseSize > 4000 ? (0, gptUtils_1.largeModel)(model) : model;
            const request = Object.assign({ model: oaiModel, messages: [
                    { role: 'system', content: systemMessage },
                    ...messages,
                ], temperature: 0, max_tokens: responseSize }, requestOverrides);
            let result = undefined;
            try {
                result = (_a = yield (0, utils_1.truthy)(this.cacher, (_) => __awaiter(this, void 0, void 0, function* () { return yield _.retrieveRequestFromCache(request, logMetaData); }))) !== null && _a !== void 0 ? _a : yield this.aiClient.createChatCompletion(request, logMetaData);
                const responseStr = (_c = (_b = result.choices[0]) === null || _b === void 0 ? void 0 : _b.message) === null || _c === void 0 ? void 0 : _c.content;
                (0, utils_1.assertIsDefined)(responseStr, 'response content not generated');
                if (!result.isFromCache) {
                    (_d = this.cacher) === null || _d === void 0 ? void 0 : _d.cacheRequest(result, request, logMetaData).then(_ignore => ({}));
                }
                yield this.recordAudit(systemMessage, messages, result.model, responseStr, request, result, actionName, undefined, Object.assign(Object.assign({}, logMetaData), { isFromCache: result.isFromCache ? 'true' : undefined }));
                return responseStr;
            }
            catch (err) {
                (_e = this.logger) === null || _e === void 0 ? void 0 : _e.error(`error performing [${actionName}]`, err);
                yield this.recordAudit(systemMessage, messages, model, "", request, result, undefined, err, logMetaData);
                throw new Error(`error performing blocking request [${actionName}]`);
            }
        });
    }
    // TODO: create tests for functions and wihtout functions
    makeStreamingRequest(systemMessage, messages, model, responseSize = 800, functions, logMetaData, requestOverrides, maxTokens = this.maxTokensPerRequest) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const { promptSize } = this.measureRequest(model, systemMessage, messages, responseSize, maxTokens);
            yield this.moderateLastMessage(messages);
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.info(`making streaming request`, logMetaData);
            const requestMessages = [
                { role: 'system', content: systemMessage },
                ...messages,
            ];
            const request = Object.assign({ model: promptSize + responseSize > 4000 ? (0, gptUtils_1.largeModel)(model) : model, messages: [
                    { role: 'system', content: systemMessage },
                    ...messages,
                ], temperature: 0, max_tokens: responseSize }, requestOverrides);
            let readContent = "";
            let streamingFunctionName = undefined;
            try {
                const response = yield this.aiClient.createStreamingChatCompletion(request, logMetaData);
                const streamTransform = new OpenAIStreamChunkTransform_1.OpenAIStreamChunkTransform(this.logger, logMetaData);
                // TODO: for streaming - cache + retrieve this from cache..
                return (0, stream_1.pipeline)(response, streamTransform, new StreamListenerTransform_1.StreamListenerTransform((x) => {
                    streamingFunctionName = x.functionName;
                    return readContent += x.chunk;
                }), err => {
                    var _a;
                    if (err) {
                        (_a = this.logger) === null || _a === void 0 ? void 0 : _a.error('error during stream', err !== null && err !== void 0 ? err : {}, logMetaData);
                    }
                    this.recordAudit(systemMessage, requestMessages, model, readContent, request, undefined, streamingFunctionName, err !== null && err !== void 0 ? err : undefined, logMetaData);
                });
            }
            catch (err) {
                (_b = this.logger) === null || _b === void 0 ? void 0 : _b.error(`error performing streaming request`, err, logMetaData);
                yield this.recordAudit(systemMessage, requestMessages, model, readContent, request, undefined, streamingFunctionName, err, logMetaData);
                return undefined;
            }
        });
    }
    performModeration(body, options, logMetaData) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!((_a = this.settings) === null || _a === void 0 ? void 0 : _a.disableModeration)) {
                const response = yield this.aiClient.performModeration(body, options);
                const isFlagged = response.results[0].flagged;
                (_b = this.logger) === null || _b === void 0 ? void 0 : _b.debug(`request flagged due to moderation. [${JSON.stringify(response.results[0])}]`, logMetaData);
                if (isFlagged) {
                    throw new Error(`request flagged due to moderation.`);
                }
            }
        });
    }
    moderateLastMessage(messages, logMetaData) {
        return __awaiter(this, void 0, void 0, function* () {
            const content = messages.slice(-1).map(_ => _.content).join('\n');
            yield this.performModeration({ input: content }, undefined, logMetaData);
        });
    }
    extractFunctionValidatedResult(readContent, functionSchema) {
        return (0, utils_1.truthy)(readContent, funcArgs => {
            const json = JSON5.parse(funcArgs, undefined, (error, stack, root) => {
                if (error.message.includes('invalid end of input')) {
                    const validator = this.schemaValidationCache.getValidator(functionSchema);
                    if (validator(root)) {
                        // we did what we could.. object looks good enough..
                        return { type: 'return-healed', value: root };
                    }
                    else {
                        throw new Error(`function arguments did not pass validation [${JSON.stringify(validator.errors)}]`);
                    }
                }
                else {
                    return undefined;
                }
            });
            const validator = this.schemaValidationCache.getValidator(functionSchema);
            if (validator(json)) {
                return json;
            }
            else {
                throw new Error(`function arguments did not pass validation [${JSON.stringify(validator.errors)}]`);
            }
        });
    }
    recordAudit(systemMessage, requestMessages, model, readContent, request, response, functionName, err, logMetaData) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const prompt_tokens = (0, gptUtils_1.countTokens)(systemMessage + requestMessages.map(_ => _.content).join('\n'), (_a = response === null || response === void 0 ? void 0 : response.model) !== null && _a !== void 0 ? _a : model, this.logger);
            const completion_tokens = (0, gptUtils_1.countTokens)(readContent, (_b = response === null || response === void 0 ? void 0 : response.model) !== null && _b !== void 0 ? _b : model, this.logger);
            yield ((_c = this.auditor) === null || _c === void 0 ? void 0 : _c.auditRequest({
                request,
                resultRaw: {
                    id: response === null || response === void 0 ? void 0 : response.id,
                    model: response === null || response === void 0 ? void 0 : response.model,
                    choices: [{
                            message: functionName
                                ? {
                                    function_call: {
                                        name: functionName,
                                        arguments: readContent
                                    }
                                }
                                : {
                                    content: readContent,
                                    role: 'assistant'
                                },
                            index: 0,
                            finish_reason: err === null || err === void 0 ? void 0 : err.message
                        }],
                    object: response === null || response === void 0 ? void 0 : response.object,
                    created: response === null || response === void 0 ? void 0 : response.created,
                    usage: {
                        prompt_tokens,
                        completion_tokens,
                        total_tokens: prompt_tokens + completion_tokens,
                    }
                },
                result: err
                    ? { error: { message: err.message, details: err.toString() } }
                    : { data: { content: readContent } },
                requestType: 'blocking-request',
                requestSig: functionName !== null && functionName !== void 0 ? functionName : '',
                metaData: logMetaData,
            }));
        });
    }
    measureRequest(model, systemMessage, messages, responseSize, maxTokens) {
        var _a;
        const promptSize = (0, gptUtils_1.countTokens)(systemMessage + messages.map(_ => _.content).join('\n'), model, this.logger);
        if (promptSize + responseSize > maxTokens) {
            const m = `warning: input too large. model [${model}] prmopt=[${promptSize}] response=[${responseSize}] max=[${maxTokens}]`;
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.error(m, {});
            throw new Error(m);
        }
        return {
            model,
            promptSize,
        };
    }
}
exports.Contractor = Contractor;
const createResultWrapper = (name, tSchema, description = "") => ({
    type: "object",
    additionalProperties: false,
    description,
    properties: {
        type: { type: "string", const: name },
        value: tSchema,
    },
    required: ['type', 'value'],
});
exports.createResultWrapper = createResultWrapper;
const createResultsWrapper = (funcs) => ({
    $id: 'results-wrapper',
    title: 'Result',
    type: "object",
    additionalProperties: false,
    oneOf: funcs.map(f => ({
        type: 'object',
        description: f.description,
        properties: {
            name: { type: 'string', const: f.name },
            value: f.parameters,
        },
    }))
});
exports.createResultsWrapper = createResultsWrapper;
