"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenAIStreamToStreamedObjectTransform = void 0;
const stream_1 = require("stream");
const utils_1 = require("./utils");
const JSON5 = __importStar(require("./json5"));
/**
 * this class transforms a stream of {OpenAIStreamObject} to {StreamedObject<T>>}
 */
class OpenAIStreamToStreamedObjectTransform extends stream_1.Transform {
    constructor(validateFunction, healPath, logger, typeValueAttribute = "type", validatoinValueAttribute) {
        super({ objectMode: true });
        this.typeValueAttribute = typeValueAttribute;
        this.validatoinValueAttribute = validatoinValueAttribute;
        this.validateFunction = validateFunction;
        this.healPath = healPath;
        this.logger = logger;
        this.jsonStream = "";
    }
    _transform(chunk, encoding, callback) {
        var _a, _b, _c;
        // Transform the data to uppercase
        const incoming = chunk;
        if (!Array.isArray(incoming)) {
            return callback(new Error(`ObjectStreamTransform was expecting array, instead got [${JSON.stringify(incoming)}]`));
        }
        (0, utils_1.truthy)((_a = incoming.at(0)) === null || _a === void 0 ? void 0 : _a.functionName, _ => this.objectTypeByFunctionName = _);
        this.jsonStream += incoming.map(_ => _.chunk).join('');
        try {
            const healedObject = JSON5.parse(this.jsonStream, undefined, (error, stack, root) => {
                var _a, _b;
                let _healPath = [];
                if (Array.isArray(this.healPath)) {
                    _healPath = this.healPath;
                }
                else {
                    this.objectTypeByFunctionName = this.objectTypeByFunctionName
                        || ((root === null || root === void 0 ? void 0 : root.hasOwnProperty(this.typeValueAttribute)) ? root[this.typeValueAttribute] : undefined);
                    if (this.objectTypeByFunctionName) {
                        // if healpath not provided for this type just return what ever works (if validation passes)
                        _healPath = (_a = this.healPath.get(this.objectTypeByFunctionName)) !== null && _a !== void 0 ? _a : [];
                    }
                    else if (!!this.objectTypeByFunctionName) {
                        (_b = this.logger) === null || _b === void 0 ? void 0 : _b.error(`warning: object type not mapped [${this.objectTypeByFunctionName}]`, error);
                    }
                }
                if (error.message.includes(`invalid character '`)) {
                    return { type: 'skip-char' };
                }
                if (error.message.includes('invalid end of input') && stack.length > _healPath.length) {
                    const healKey = _healPath.at(-1);
                    if (healKey) {
                        (0, utils_1.assertIsDefined)(healKey);
                        const healStackPath = stack[_healPath.length - 1];
                        if (healStackPath === null || healStackPath === void 0 ? void 0 : healStackPath.hasOwnProperty(healKey)) {
                            if (healStackPath[healKey] === stack[_healPath.length]) {
                                if (Array.isArray(healStackPath[healKey])) {
                                    healStackPath[healKey].pop();
                                }
                            }
                        }
                        return { type: 'return-healed', value: root };
                    }
                }
            });
            const validationFunction = (this.validateFunction instanceof Map
                ? (this.objectTypeByFunctionName ? this.validateFunction.get(this.objectTypeByFunctionName) : undefined)
                : this.validateFunction);
            if (!validationFunction) {
                return callback(new Error(`did not find suitable validation function [${this.objectTypeByFunctionName}]`));
            }
            else {
                const validated = validationFunction(this.validatoinValueAttribute
                    ? healedObject[this.validatoinValueAttribute]
                    : healedObject);
                if (validated) {
                    this.push(JSON.stringify({
                        functionName: this.objectTypeByFunctionName,
                        stream: healedObject,
                    }) + '\n');
                }
            }
            callback();
        }
        catch (err) {
            if ((_b = err.message) === null || _b === void 0 ? void 0 : _b.includes('invalid end of input')) {
                // ignore while streaming...
                callback();
            }
            else {
                (_c = this.logger) === null || _c === void 0 ? void 0 : _c.error('error caught', err);
                callback(new Error('caught error:' + err.message));
            }
        }
    }
}
exports.OpenAIStreamToStreamedObjectTransform = OpenAIStreamToStreamedObjectTransform;
