"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenAIStreamTransform = void 0;
const stream_1 = require("stream");
const JSON5 = __importStar(require("./json5"));
/**
 * extracts object from OpenAI lines format ("data: ...")
 *  transforms this into {OpenAIStreamObject}
 **/
class OpenAIStreamTransform extends stream_1.Transform {
    constructor(logger, logMetaData) {
        super({ objectMode: true });
        this.functionName = undefined;
        // sometimes a non-complete chunk of the steam comes.. we keep it here for the next drop to prepend to..
        this.partialChunk = undefined;
        this.logger = logger;
        this.logMetaData = logMetaData;
    }
    _transform(chunk, encoding, callback) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        // Transform the data to uppercase
        let processChunk = chunk.toString();
        if (this.partialChunk) {
            processChunk = this.partialChunk + processChunk;
            this.partialChunk = undefined;
        }
        const payloads = processChunk.toString().split("\n\n");
        for (const payload of payloads) {
            if (payload.trim() === 'data: [DONE]') {
                // the chat stream is done..
            }
            else if (payload.startsWith("data:")) {
                const data = payload.replaceAll(/(\n)?^data:\s*/g, ''); // in case there's multiline data event
                try {
                    const delta = JSON5.parse(data.trim());
                    const funcName = (_c = (_b = (_a = delta.choices[0]) === null || _a === void 0 ? void 0 : _a.delta) === null || _b === void 0 ? void 0 : _b.function_call) === null || _c === void 0 ? void 0 : _c.name;
                    if (funcName) {
                        this.functionName = funcName;
                    }
                    const content = (_f = (_e = (_d = delta.choices[0]) === null || _d === void 0 ? void 0 : _d.delta) === null || _e === void 0 ? void 0 : _e.content) !== null && _f !== void 0 ? _f : (_j = (_h = (_g = delta.choices[0]) === null || _g === void 0 ? void 0 : _g.delta) === null || _h === void 0 ? void 0 : _h.function_call) === null || _j === void 0 ? void 0 : _j.arguments;
                    if (content) {
                        this.push({
                            functionName: this.functionName,
                            chunk: content || ''
                        });
                    }
                }
                catch (error) {
                    // sometimes OpenAI sends us only partial json..
                    if (error.toString().trim().includes('invalid end of input') || error.toString().trim().includes(`Cannot read properties of undefined (reading '0')`)) {
                        this.partialChunk = chunk;
                        return callback();
                    }
                    else {
                        (_k = this.logger) === null || _k === void 0 ? void 0 : _k.error(`Error with JSON5.parse payload=[${payload}].\n${error}`, error, this.logMetaData);
                        return callback(new Error('error processing response, please try again'));
                    }
                }
            }
        }
        // Call the callback to indicate that the transformation is complete
        callback();
    }
}
exports.OpenAIStreamTransform = OpenAIStreamTransform;
