import { Logger } from "./Logger";
export type GPTModels = 'gpt-3.5-turbo-0613' | 'gpt-4-1106-preview' | 'gpt-3.5-turbo-16k-0613' | 'gpt-3.5-turbo-1106' | 'gpt-4-turbo-2024-04-09' | string;
export declare const largeModel: (model: GPTModels) => GPTModels;
export declare function countTokens(input: string, model: GPTModels, logger?: Logger): number;
export declare function truncateInput(input: string, model: GPTModels, maxSize: number, logger?: Logger): string;
type Vector = number[];
/**
 * Combine two vectors by applying weights and adding them together.
 *
 * @param {Vector} v1 - The first vector to combine. This is typically the category embedding vector.
 * @param {Vector} v2 - The second vector to combine. This is typically the keyword embedding vector.
 * @param {number} w1 - The weight to apply to the first vector (v1). This must be between 0 and 1.
 *
 * @returns {Vector} The combined vector.
 *
 * @throws {Error} If the two vectors have different lengths.
 *
 * @typedef {number[]} Vector - A vector is represented as an array of numbers.
 *
 * This function takes two vectors and a weight for the first vector, and combines them into a single vector.
 * The vectors should be the same length, and they should typically be embedding vectors produced by a machine learning model.
 * The weight for the second vector is implicitly set to be (1 - w1), so the weights for the two vectors always add up to 1.
 */
export declare function combineVectors(v1: Vector, v2: Vector, w1: number): Vector;
export {};
