"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.truthy = exports.retryPromise = exports.getRandomInt = exports.delayedPromise = exports.stringToHashKey = exports.toBase64 = exports.assertIsDefined = void 0;
const spark_md5_1 = __importDefault(require("spark-md5"));
function assertIsDefined(value, errorMsg = 'value was not defined') {
    if (value === null || value === undefined) {
        throw Error(errorMsg);
    }
}
exports.assertIsDefined = assertIsDefined;
const toBinary = (string) => {
    const codeUnits = Uint16Array.from({ length: string.length }, (element, index) => string.charCodeAt(index));
    const charCodes = new Uint8Array(codeUnits.buffer);
    let result = "";
    charCodes.forEach((char) => {
        result += String.fromCharCode(char);
    });
    return result;
};
const toBase64 = (input) => btoa(toBinary(input));
exports.toBase64 = toBase64;
const stringToHashKey = (input) => (0, exports.toBase64)(spark_md5_1.default.hash(input).toString()).slice(0, 15);
exports.stringToHashKey = stringToHashKey;
const delayedPromise = (delayMS, action) => new Promise((resolve, reject) => {
    const timer = setTimeout(() => {
        clearTimeout(timer);
        try {
            resolve(action());
        }
        catch (e) {
            reject(e);
        }
    }, delayMS);
});
exports.delayedPromise = delayedPromise;
const getRandomInt = (max) => Math.floor(Math.random() * Math.floor(max));
exports.getRandomInt = getRandomInt;
const _retryPromise = (name, action, maxRetries, attempt = 0, ignoreErrors) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        return yield action();
    }
    catch (e) {
        if (ignoreErrors && ignoreErrors(e)) {
            throw e;
        }
        else if (attempt >= maxRetries) {
            console.error(`max retries reached [${name}] error=[${String(e)}] (original error=[${String(e)}])`);
            throw e;
        }
        else {
            return yield (0, exports.delayedPromise)(1000 + (0, exports.getRandomInt)(300), () => {
                console.debug(`_retryPromise [${name}] retrying..`);
                return _retryPromise(name, action, maxRetries, attempt + 1, ignoreErrors);
            });
        }
    }
});
const retryPromise = (name, action, maxRetries, ignoreErrors) => _retryPromise(name, action, maxRetries, 0, ignoreErrors);
exports.retryPromise = retryPromise;
const truthy = (t, action) => (t !== null && t !== undefined && t !== false && action(t) || undefined) || undefined;
exports.truthy = truthy;
